// ===========================================================================
//
// Licensed Materials - Property of IBM
//
// 5639-B43
//
// (c) Copyright IBM Corp. 2002
//
// ===========================================================================
// Change Activity:
// CFD List:
//  pn Reason    Rlse   Date     Origin  Comments
// --- --------  ----   ------   ------  ------------------------------------
//                      011127   HDMIR:  Create new JMS applet sample
// End CFD List.
// Inspected:
// End Change Activity.
//
// ===========================================================================
/*
    This class is a basic extension of the Applet class.  It would generally be
    used as the main class with a Java browser or the AppletViewer.  But an instance
    can be added to a subclass of Container.  To use this applet with a browser or
    the AppletViewer, create an html file with the following code:

    <HTML>
    <HEAD>
    <TITLE>WebSphere MQ JMS Applet Client</TITLE>
    </HEAD>
    <BODY>

    <APPLET CODE="JMSTestApplet.class" CODEBASE=".."
        ARCHIVE="jms.jar,com.ibm.mq.jar,com.ibm.mqjms.jar,connector.jar,jta.jar,jmsapplet.properties"
        WIDTH=600 HEIGHT=300 align=center>
    <PARAM name="queueManager" value="your.queue.manager">
    <PARAM name="hostName" value="your.hostname">
    <P><b><i>
    <p>Applet should appear here.</p>
    <p>Sorry, your browser does not support Java applets:</p>
    <p>You must use a browser that can support Java applets in order to run the
    WebSphere MQ/JMS Applet Client.</p>
    </b></i>
    </APPLET> 

    </BODY>

    </HTML>
    
    
    The interesting part of the applet occurs within the clickedConnect() method
 
*/

import javax.swing.*;
import java.net.*;
import java.util.*;
import java.text.*;
import java.awt.*;
import java.io.*;
import java.awt.event.*;

// The MQ / JMS libraries
import com.ibm.mq.*;
import com.ibm.mq.jms.*;
import javax.jms.*;


/**
 * Applet that shows the use of MQJMS from within a browser.<BR>
 * Creation date: (27/11/2001 11:13:57)
 * @author: HDMIR
 */
public class JMSTestApplet extends javax.swing.JApplet
implements MouseListener, WindowListener
{

  /** GUI: output pane */
  private javax.swing.JTextPane output = null;
  /** GUI: connect button */
  private javax.swing.JButton connectButton = null;
  /** GUI: Label for the queue manager field */
  private javax.swing.JLabel qMgrNameLabel = null;
  /** GUI: Scrollpane for the output window */
  private javax.swing.JScrollPane outputScrollPane = null;
  /** GUI: host Name */
  private javax.swing.JTextField hostName = null;
  /** GUI: Label for the host name field */
  private javax.swing.JLabel hostNameLabel = null;
  /** GUI: queue manager name */
  private javax.swing.JTextField qMgrName = null;

  /** The channel to be used */
  private String channel = "SYSTEM.DEF.SVRCONN";
  /** The queue to be accessed */
  private String queue = "SYSTEM.DEFAULT.LOCAL.QUEUE"; 
  /** The name of the resource bundle to load */
  private String bundleName = "jmsapplet";


  /** The contents of the message resource bundle, to display NLS messages */
  protected static ResourceBundle messages = null;
  /** Flag to show whether the catalog was successfully loaded */
  protected static boolean messageCatalogueLoaded = false;


  /**
    * Default constructor that calls the superclass constructor.
    */
  public JMSTestApplet()
  {
    super();

  }// constructor


  /**
    * Gets the applet information.
    * @return java.lang.String
    */
  public java.lang.String getAppletInfo()
  {
    return "JMSTestApplet.class\n(c) Copyright IBM Corp. 2002";

  }// getAppletInfo


  /**
    * Outputs the supplied string to the user display. This method allows easy
    * switching between the standard out stream, and the gui output box for
    * debugging.
    * @param data java.lang.String
    */
  public void appendOutput(String data)
  {
    // Append the supplied data to the GUI.
    output.setText(output.getText()+"\n"+data);
    output.repaint();

    // Pass the data string to standard out.
    //System.out.println(data);

  }// appendOutput


  /**
    * This method is called when the connect button is pressed. It creates a 
    * new thread to carry out the connection test without affecting the user
    * interface.
    */
  public void clickedConnect()
  {

    // Create a new thread
    Thread test = new Thread(new ConnectionTest(this));

    // Start the connection test. The start method will bring about the
    // disabling of the connect button for the duration of the test.
    test.start();
                

  }// clickedConnect


  // **********************************************************************


  /**
    * Retrieve a message from the message catalog for use in the GUI.
    * @param msgId The identifier of the message to be returned.
    * @return String The requested message.
    */
  protected String getMessage(int msgId)
  {
    if (messages != null)
    {
      String msg = messages.getString(Integer.toString(msgId));
      return msg;
    } else return "Unable to retrieve message from catalog";

  }// getMessage(int)


  /**
    * Retrieve a message from the message catalog for use in the GUI.
    * Allows the insertion of one string (to replace the {0} text)
    * @param msgId The identifier of the message to be returned.
    * @param insert The string to replace {0} with in the message.
    * @return String The requested message.
    */
  protected String getMessage(int msgId, int insert)
  {
    if (messages != null)
    {
      String msg = messages.getString(Integer.toString(msgId));
      String[] inserts = new String[1];
      inserts[0] = Integer.toString(insert);
      return MessageFormat.format(msg,inserts);
    } else return "Unable to retrieve message from catalog";

  }// getMessage(int, int)


  /**
    * Retrieve a message from the message catalog for use in the GUI.
    * Allows the insertion of one string (to replace the {0} text)
    * @param msgId The identifier of the message to be returned.
    * @param insert The string to replace {0} with in the message.
    * @return String The requested message.
    */
  protected String getMessage(int msgId, String insert)
  {
    if (messages != null)
    {
      String msg = messages.getString(Integer.toString(msgId));
      String[] inserts = new String[1];
      inserts[0] = insert;
      return MessageFormat.format(msg,inserts);
    } else return "Unable to retrieve message from catalog";

  }// getMessage (int, String)


  /**
    * Retrieve a message from the message catalog for use in the GUI.
    * Allows the insertion of one string (to replace the {0} text)
    * @param msgId The identifier of the message to be returned.
    * @param insert1 The string to replace {0} with in the message.
    * @param insert2 The string to replace {1} with in the message.
    * @param insert3 The string to replace {2} with in the message.
    * @return String The requested message.
    */
  protected String getMessage(int msgId, String insert1,String insert2,
                              int insert3)
  {
    if (messages != null)
    {
      String msg = messages.getString(Integer.toString(msgId));
      String[] inserts = new String[3];
      inserts[0] = insert1;
      inserts[1] = insert2;
      inserts[2] = Integer.toString(insert3);
      return MessageFormat.format(msg,inserts);
    } else return "Unable to retrieve message from catalog";

  }// getMessage(int, String, String, int)


  // **********************************************************************
  // Below this point we have various helper classes and methods, including
  // the methods from MouseListener and WindowListener, placed here to avoid
  // creating several inner classes.

  /**
    * The mouse was clicked over the applet
    * @param java.awt.event.MouseEvent The mouse action that took place.
    */
  public void mouseClicked(java.awt.event.MouseEvent e)
  {
    // If the connect button was pressed, then run the method
    if (e.getSource() == connectButton) clickedConnect();

  };
  public void mouseEntered(java.awt.event.MouseEvent e)
  {
  };
  public void mouseExited(java.awt.event.MouseEvent e)
  {
  };
  public void mousePressed(java.awt.event.MouseEvent e)
  {
  };
  public void mouseReleased(java.awt.event.MouseEvent e)
  {
  };


  // The window listener methods
  public void windowClosing(java.awt.event.WindowEvent e)
  {
    System.exit(0);
  };
  public void windowOpened(java.awt.event.WindowEvent e)
  {
  };
  public void windowClosed(java.awt.event.WindowEvent e)
  {
  };
  public void windowIconified(java.awt.event.WindowEvent e)
  {
  };
  public void windowDeiconified(java.awt.event.WindowEvent e)
  {
  };
  public void windowActivated(java.awt.event.WindowEvent e)
  {
  };
  public void windowDeactivated(java.awt.event.WindowEvent e)
  {
  };

  /**
    * Handle the Applet init method.
    */
  public void init()
  {
    try
    {
      // Set the size of the applet
      setSize(600, 300);
      //setContentPane(getJAppletContentPane());

      // Load the message catalog
      initCatalog();

      // Set up the gui controls, including the label names taken
      // from the message catalog.
      initControls();

      // Connect the button listener
      connectButton.addMouseListener(this);
      
    } catch (java.lang.Throwable ivjExc)
    {
      // Something may have gone wrong here - ignore it!

    }// try 

  }// init


  /**
    * Tries to read in the message catalog in order to provide
    * NLS messages
    */
  private void initCatalog()
  {

    try
    {

      // Load the resourcebundle
      messages = ResourceBundle.getBundle(bundleName);

      // Mark it as having been successfully loaded.
      messageCatalogueLoaded = true;

    } catch (MissingResourceException ex)
    {
      // There was a problem loading the resource bundle.
      messageCatalogueLoaded = false;
      System.out.println("Failed to load resource bundle "+bundleName);

    } catch (java.lang.Error err)
    {
      // A java type of error occured.
      messageCatalogueLoaded = false;
      System.out.println("Failed to load resource bundle "+bundleName);

    }// try 

  }// initCatalog


  /**
    * Handles the setup of the GUI controls, instantiating the objects, and
    * loading the text from the message catalog.
    */
  private void initControls()
  {

    // The connect button
    connectButton = new javax.swing.JButton(getMessage(2));
    connectButton.setBounds(253, 69, 194, 25);

    // The qmgr label
    qMgrNameLabel = new javax.swing.JLabel(getMessage(7));
    qMgrNameLabel.setBounds(13, 45, 232, 14);
    qMgrNameLabel.setHorizontalTextPosition(javax.swing.SwingConstants.LEFT);
    qMgrNameLabel.setHorizontalAlignment(javax.swing.SwingConstants.RIGHT);

    // Host name label
    hostNameLabel = new javax.swing.JLabel(getMessage(4));
    hostNameLabel.setBounds(13, 18, 232, 14);
    hostNameLabel.setHorizontalTextPosition(javax.swing.SwingConstants.LEFT);
    hostNameLabel.setHorizontalAlignment(javax.swing.SwingConstants.RIGHT);

    // The output pane
    output = new javax.swing.JTextPane();
    output.setPreferredSize(new java.awt.Dimension(1600, 200));
    output.setBounds(0, 0, 1622, 173);

    // The output pane scrollpane
    outputScrollPane = new javax.swing.JScrollPane(output);
    outputScrollPane.setVerticalScrollBarPolicy(javax.swing.JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
    outputScrollPane.setBounds(10, 111, 574, 177);

    // Host name
    hostName = new javax.swing.JTextField();
    if (getParameter("hostName") != null) hostName.setText(getParameter("hostName"));
    hostName.setBounds(254, 15, 194, 20);

    // Get the codebase of the applet
    URL codebase = getCodeBase();
    if ((codebase.getHost() != null) && (codebase.getHost().length() != 0))
    {
      // Set the hostname for the user.
      hostName.setText(codebase.getHost());

      // If we are downloading this off a website, then the user may only
      // connect to the host from which it came.
      hostName.setEnabled(false); 

    }// if

    // qMgr name
    qMgrName = new javax.swing.JTextField();
    if (getParameter("queueManager") != null) qMgrName.setText(getParameter("queueManager"));
    qMgrName.setBounds(254, 42, 194, 20);
    
    // Set up the content pane to contain all the controls.
    ((JPanel)getContentPane()).setPreferredSize(new java.awt.Dimension(500, 260));
    getContentPane().setLayout(null);
    ((JPanel)getContentPane()).setMaximumSize(new java.awt.Dimension(0, 0));
    getContentPane().add(qMgrName);
    getContentPane().add(qMgrNameLabel);
    getContentPane().add(connectButton);
    getContentPane().add(outputScrollPane);
    getContentPane().add(hostName);
    getContentPane().add(hostNameLabel);


  }// initControls

  /**
    * Main method to allow applet to be executed as an application.
    * @param args java.lang.String[]
    */
  public static void main(java.lang.String[] args)
  {
    try
    {

      // Create the required objects
      javax.swing.JFrame frame = new javax.swing.JFrame();
      JMSTestApplet aJMSTestApplet;
      
      // Instantiate the class
      Class iiCls = Class.forName("JMSTestApplet");
      ClassLoader iiClsLoader = iiCls.getClassLoader();
      aJMSTestApplet = (JMSTestApplet)java.beans.Beans.instantiate(iiClsLoader,"JMSTestApplet");

      // Add the applet to the content pane
      frame.getContentPane().add("Center", aJMSTestApplet);
      
      // Start the applet
      aJMSTestApplet.start();

      // Set the window title and size
      frame.setTitle("JMS Applet Client (running as application)");
      frame.setSize(aJMSTestApplet.getWidth(),
                    aJMSTestApplet.getHeight() + 30);

      // Add a listener for the frame closing
      frame.addWindowListener(aJMSTestApplet);
      
      // Make the frame visible
      frame.setVisible(true);

    } catch (Throwable exception)
    {
      System.err.println("Exception occurred in main() of JMSTestApplet");
      exception.printStackTrace(System.out);
    }
  }  


  // **********************************************************************

  /**
    * Carries out the network traffic part of the applet in a seprate thread
    * to allow the user interface to update correctly.
    * Creation date: (27/11/2001 11:13:57)
    * @author: HDMIR
    */
  class ConnectionTest implements Runnable
  {
    /** The GUI object, stored so we can output to the window */
    private JMSTestApplet gui = null;

    /**
      * Constructor
      * @param theGUI A reference to the Applet window
      */
    public ConnectionTest(JMSTestApplet theGUI)
    {
      // Store the GUI for later use
      gui = theGUI;
    }// constructor


    /**
      * The code that actually carries out the connection test.
      */
    public void run()
    {

      // Disable the gui button for the duration of the test
      setButtonEnabled(false);

      // Catch all the exceptions we might expect
      try
      {
        // Clear the GUI output
        output.setText("");

        // Print the header message
        gui.appendOutput(getMessage(1));

        try
        {

          // Check that the MQ libraries have loaded
          Class.forName("com.ibm.mq.jms.services.ConfigEnvironment");
          Class.forName("com.ibm.mq.MQEnvironment");

        } catch (ClassNotFoundException e)
        {

          // We are unable to load the MQ libraries
          gui.appendOutput(getMessage(3));
          setButtonEnabled(true);
          return;

        } catch (NoClassDefFoundError e)
        {
          // The web browser cannot find the classes
          gui.appendOutput(getMessage(34));
          setButtonEnabled(true);
          return;
        }


        // Store the host name in a string
        String hostn = hostName.getText().trim();

        // Check whether a hostname has been supplied.
        if (hostn.equals(""))
        {
          // No hostname supplied message
          gui.appendOutput(getMessage(36));
          setButtonEnabled(true);
          return;
        }

        // About to check for host message
        gui.appendOutput(getMessage(37));

        try
        {
          // Test that we can connect to the specified host.
          Socket s = new Socket(hostn, MQEnvironment.port);
          s.close();

        } catch (SecurityException ex)
        {
          // Browser will not grant permission to this host
          gui.appendOutput(getMessage(38,hostn));
          setButtonEnabled(true);
          return;

        } catch (UnknownHostException ex)
        {
          // Host is unknown
          gui.appendOutput(getMessage(39,hostn));
          setButtonEnabled(true);
          return;

        } catch (IOException ex)
        {
          // Listener is probably not running
          gui.appendOutput(getMessage(40,hostn, hostn,
                                  MQEnvironment.port));
          setButtonEnabled(true);
          return;

        }// try 

        // Successfully accessed host message
        gui.appendOutput(getMessage(41,hostn));

        // Create a QCF
        MQQueueConnectionFactory myMQ_QCF = new MQQueueConnectionFactory();

        // If we have a hostname to connect to...
        if (hostn.length() != 0)
        {
          // Set up the client connect information.
          myMQ_QCF.setHostName(hostn);
          myMQ_QCF.setTransportType(com.ibm.mq.jms.JMSC.MQJMS_TP_CLIENT_MQ_TCPIP);
          myMQ_QCF.setChannel(channel);

          // Give suitable user output
          gui.appendOutput(getMessage(4)+" "+hostn);
          gui.appendOutput(getMessage(6)+" "+channel);

        } else
        {
          // No host name supplied. Give error message and terminate.
          // NB This has already been checked, and so should never be reached.
          gui.appendOutput(getMessage(36));
          setButtonEnabled(true);
          return;
        }

        // Set the queue manager on the QCF and output a user message
        myMQ_QCF.setQueueManager(qMgrName.getText());
        gui.appendOutput(getMessage(7)+" "+qMgrName.getText());

        // Give a warning message that you may have to wait for the
        // connection to time out.
        gui.appendOutput(getMessage(57));

        // Create a queue connection
        QueueConnection qConnect = myMQ_QCF.createQueueConnection();

        // Start the connection
        qConnect.start();

        // Create a new Qsession using the connection.
        QueueSession qSession = qConnect.createQueueSession(false, Session.AUTO_ACKNOWLEDGE);

        // Create the Q object, with a user message
        Queue myQ = qSession.createQueue(queue);
        gui.appendOutput(getMessage(8)+" "+queue);

        // Create a queue sender so that we can send messages about.
        QueueSender qSender = qSession.createSender(myQ);

        // Create a text message.
        String theMessage = "MQJMS Applet Client - Sample message";
        Message testMessage = qSession.createTextMessage(theMessage);

        // Create the subscriber here so it can collect the message.
        QueueReceiver qReceiver = qSession.createReceiver(myQ);

        // Send the message
        qSender.send(testMessage);

        // Receive back, and output the message
        TextMessage receiveOne = (TextMessage)qReceiver.receive();
        gui.appendOutput(getMessage(27));

        // Retrieve the message text
        String received = receiveOne.getText();

        // Check that it is the same as the message that was sent.
        if (received.equals(theMessage))
        {
          // SUCCESS! message
          gui.appendOutput(getMessage(55));

        } else
        {
          // Received a different message.
          gui.appendOutput(getMessage(56));

        }// if 

      } catch (JMSException e)
      {

        // Get the linked exception
        Exception el = e.getLinkedException();

        // If it is an MQException (which it should be)
        if (el instanceof MQException)
        {
          MQException mqe = (MQException)el;
          gui.appendOutput("");

          // Output an error message, and show the details
          gui.appendOutput(getMessage(9));
          switch (mqe.reasonCode)
          {
          case 2009: // Connection to qmgr was lost
            gui.appendOutput(getMessage(10));
            break;
          case 2059: // qmgr not available
            gui.appendOutput(getMessage(11));
            break;
          case 2058: // invalid qmgr name
            gui.appendOutput(getMessage(12));
            break;
          case 2063: // security error
            gui.appendOutput(getMessage(13));
            break;
          case 2085: // no system.default.local.queue present
            gui.appendOutput(getMessage(17));
            break;

          case 2030: // max message length too small
            gui.appendOutput(getMessage(21));
            break;
          case 2053: // queue is full
            gui.appendOutput(getMessage(22));
            break;

          case 2033: // no message available
            gui.appendOutput(getMessage(25));
            break;

          default: // refer to manual.
            gui.appendOutput(getMessage(14,mqe.reasonCode));
          }// switch

        } else
        {
          // This is not an MQ exception. Leave it alone.

        } // if


      } catch (Exception e)
      {
        // Print the exception to the output window
        gui.appendOutput(e.toString());

      }// try

      // Enable the GUI button
      setButtonEnabled(true);

    }// run


    /**
      * Enable the gui button
      * @param Whether the button should be enabled or not
      */
    private void setButtonEnabled(boolean value)
    {

      if (value)
      {
        gui.connectButton.setEnabled(true);
        gui.connectButton.setText(getMessage(2));
      } else
      {
        gui.connectButton.setEnabled(false);
        gui.connectButton.setText(getMessage(58));
      }

    }// enableButton


  }// ConnectionTest class


}// JMSTestApplet
